/*
 * File:
 *   common.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Common definitions.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef COMMON_H
#define COMMON_H

/* ################################################################### *
 * DEFINES
 * ################################################################### */

/* Prefixes */
#define FILE_PREFIX             "file://"
#define SSH_PREFIX              "ssh://"
#define TCP_PREFIX              "tcp://"

#define COMMENT_PREFIX          "#"

/* Protocol configuration */
#define DEGREE_CONFIG           "degree="

/* Limits */
#define MAX_PATH                256
#define MAX_HOST                128
#define MAX_PEERS               10000
#define MAX_ARGS                512
#define BUF_SIZE                2048
#define FLUX_LIST_SIZE          256

/* Defaults */
#define DEFAULT_VERBOSITY       3
#define DEFAULT_CHUNK_SIZE      (16*1024)
#define DEFAULT_TIMEOUT         60
#define DEFAULT_LOCAL_HOST      "localhost"
#define DEFAULT_LOCAL_PORT      9999
#define DEFAULT_PROTOCOL        "parallel"
#define DEFAULT_PROGRAM         "crcp"

#ifndef MIN
#define MIN(a,b)                ((a) < (b) ? (a) : (b))
#endif
#ifndef MAX
#define MAX(a,b)                ((a) > (b) ? (a) : (b))
#endif

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

typedef struct config {
  int verbosity;
  int local_port;
  const char *local_host;
  int chunk_size;
  int timeout;
  const char *program;
  const char *protocol;
  char *args;
} config_t;

#endif /* COMMON_H */
