/*
 * File:
 *   file.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Represents a file.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef FILE_H
#define FILE_H

#include "common.h"

/* ################################################################### *
 * DEFINES
 * ################################################################### */

#define OPEN_READ               0
#define OPEN_WRITE              1

#define TOTAL_CHUNKS(f)         (((f)->file_size + (f)->chunk_size - 1) / (f)->chunk_size)

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

typedef struct file {
  unsigned long flux_id;
  char *file_name;
  unsigned int file_size;
  unsigned int chunk_size;
  int fd;
} file_t;

typedef struct file_stats {
  unsigned int total_chunks; /* How many chunks does the file contain? */
  unsigned int nb_chunks;    /* How many chunks do we have? */
  unsigned char *chunk_mask; /* Which chunks do we have? */
} file_stats_t;

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

/* File I/O */

int open_file(file_t *f, int mode);

void close_file(file_t *f);

int read_chunk(file_t *f, unsigned int chunk, unsigned char *buf, int buflen);

int write_chunk(file_t *f, unsigned int chunk, unsigned char *buf, int buflen);

/* File statistics */

int create_stats(file_stats_t *fs, file_t* f);

void free_stats(file_stats_t *fs);

void set_chunk(file_stats_t *fs, unsigned int chunk);

void clear_chunk(file_stats_t *fs, unsigned int chunk);

void set_all(file_stats_t *fs);

void clear_all(file_stats_t *fs);

int has_chunk(file_stats_t *fs, unsigned int chunk);

#endif /* FILE_H */
