/*
 * File:
 *   log.c
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Logging management.
 *
 * Copyright (c) 2004-2005.
 */

#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#include "log.h"

/* ################################################################### *
 * VARIABLES
 * ################################################################### */

static int log_level_;

const char *LEVELS[] = {
  "",
  "[FATAL] ",
  "[ERROR] ",
  "[WARN]  ",
  "[INFO]  ",
  "[DEBUG] "
};

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

/* ======================================================================
 * log_set_verbosity
 */
void log_set_verbosity(int level)
{
  log_level_ = level;
}

/* ======================================================================
 * log_get_verbosity
 */
int log_get_verbosity()
{
  return log_level_;
}

/* ======================================================================
 * log
 */
void logm(int level, const char *fmt, ...)
{
  if (level <= log_level_) {
    va_list ap;
    va_start(ap, fmt);
    fprintf(stderr, LEVELS[level]);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
  }
}

/* ======================================================================
 * xlog
 */
void xlogm(int level, const char *fmt, ...)
{
  if (level <= log_level_) {
    va_list ap;
    va_start(ap, fmt);
    fprintf(stderr, LEVELS[level]);
    vfprintf(stderr, fmt, ap);
    va_end(ap);
    fprintf(stderr, "%s\n", strerror(errno));
  }
}

/* ======================================================================
 * fatal
 */
void fatal(int eval, const char *fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  fprintf(stderr, LEVELS[LOG_FATAL]);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  exit(eval);
}

/* ======================================================================
 * xfatal
 */
void xfatal(int eval, const char *fmt, ...)
{
  va_list ap;
  va_start(ap, fmt);
  fprintf(stderr, LEVELS[LOG_FATAL]);
  vfprintf(stderr, fmt, ap);
  va_end(ap);
  fprintf(stderr, "%s\n", strerror(errno));
  exit(eval);
}
