/*
 * File:
 *   message.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Message management.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef MESSAGE_H
#define MESSAGE_H

#include <rpc/rpc.h>

#include "common.h"

/* ################################################################### *
 * DEFINES
 * ################################################################### */

#define STATUS_OK               0
#define STATUS_ERROR            1

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

typedef enum {
  MSG_TYPE_INIT,
  MSG_TYPE_COMPLETE,
  MSG_TYPE_SEND_CHUNK,
  MSG_TYPE_REQUEST_CHUNK,
  MSG_TYPE_CONNECT_PEERS,
  MSG_TYPE_RESPONSE,
  MSG_TYPE_SHUTDOWN
} msg_type_t;

typedef struct msg_init {
  char *file_name;
  unsigned int file_size;
  unsigned int chunk_size;
  unsigned int protocol;
  char *args;
} msg_init_t;

typedef struct msg_send_chunk {
  unsigned int chunk_id;
} msg_send_chunk_t;

typedef struct msg_complete {
  double total_time;
  double chunk_time_average;
  double chunk_time_variance;
} msg_complete_t;

typedef struct msg_request_chunk {
  unsigned int chunk_id;
} msg_request_chunk_t;

typedef struct msg_peer_address {
  char *host;
  int port;
} msg_peer_address_t;

typedef struct msg_connect_peers {
  unsigned int nb;
  msg_peer_address_t *peers;
} msg_connect_peers_t;

typedef struct msg_response {
  int status;
} msg_response_t;

typedef struct msg_shutdown {
  int status;
} msg_shutdown_t;

typedef struct msg {
  unsigned long flux_id;
  msg_type_t type;
  /* Header */
  union {
    msg_init_t init;
    msg_complete_t complete;
    msg_send_chunk_t send_chunk;
    msg_request_chunk_t request_chunk;
    msg_connect_peers_t connect_peers;
    msg_response_t response;
    msg_shutdown_t shutdown;
  } h;
  /* Optional data */
  unsigned int l;
  unsigned char *d;
  /* Message chaining */
  struct msg *next;
} msg_t;

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

int msg2xdr(msg_t *msg, u_char *buf, uint32_t *len);

int xdr2msg(msg_t *msg, u_char *buf, uint32_t len);

#endif /* MESSAGE_H */
