/*
 * File:
 *   peer.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Represents a peer.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef PEER_H
#define PEER_H

#include "common.h"
#include "file.h"
#include "message.h"
#include "network.h"

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

struct config;
struct protocol;
struct flux;

/* Connection to a remote peer (in the context of one flux) */
typedef struct peer {
  connection_t conn;            /* Network connection (bi-directional) */
  struct flux *flux;            /* Flux to which this connection belongs */
  io_timer_t timer;             /* One can associate timeouts with connections */
  void *private;                /* Private storage for the protocol */
} peer_t;

/* A flux representing the distribution of one file or data stream */
typedef struct flux {
  file_t file;                  /* File read from/written to */
  file_stats_t stats;           /* Which blocks do we have (all at the source) */
  io_timer_t timer;             /* One can associate timeouts with fluxes */
  struct protocol *protocol;    /* Protocol used for distribution of this flux */
  char *args;                   /* Arguments passed to the protocol */
  peer_t *src;                  /* Connection to the source (NULL at the source) */
  int nb_peers;                 /* Number of peers active in the context of this flux */
  void *private;                /* Private storage for the protocol */
} flux_t;

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

void init_peer(struct config *config);

void terminate_peer();

peer_t *create_peer(const char *host, int port, flux_t *flux);

void init_watchdog(struct config *config);

void terminate_watchdog();

void reset_watchdog();

#endif /* PEER_H */
