/*
 * File:
 *   protocol.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Implements the distribution protocol.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef PROTOCOL_H
#define PROTOCOL_H

/* ################################################################### *
 * DEFINES
 * ################################################################### */

#define ERROR_DISCONNECT        0
#define ERROR_RECEIVE           2
#define ERROR_SEND              3

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

struct config;
struct destination;
struct file;
struct flux;
struct msg;
struct msg_peer_address;
struct peer;
struct samples;

typedef struct protocol {
  const char *name;
  struct {
    void (*init)(struct flux *, struct destination **, int);
    void (*start)(struct flux *);
    void (*terminate)(struct flux *);
    void (*error)(struct flux *, struct destination *, int);
    int (*deliver)(struct flux *, struct msg *, struct destination *);
    int (*sent)(struct flux *, struct msg *, struct destination *);
    int (*timeout)(struct flux *, struct destination *);
  } source;
  struct {
    void (*init)(struct flux *);
    void (*terminate)(struct flux *);
    void (*error)(struct flux *, struct peer *, int);
    int (*deliver)(struct flux *, struct msg *, struct peer *);
    int (*sent)(struct flux *, struct msg *, struct peer *);
    int (*timeout)(struct flux *, struct peer *);
  } peer;
} protocol_t;

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

protocol_t *get_protocol_by_name(const char *name);

protocol_t *get_protocol_by_id(int id);

int msg_init(struct peer *peer, struct file *f, const char *path, protocol_t *protocol, const char *args);

int msg_complete(struct peer *peer, file_t *f, double total_time, struct samples *chunk_time);

int msg_send_chunk(struct peer *peer, struct file *f, unsigned int chunk);

int msg_request_chunk(struct peer *peer, struct file *f, unsigned int chunk);

int msg_connect_peers(struct peer *peer, struct file *f, struct msg_peer_address *peers, int nb);

int msg_response(struct peer *peer, struct file *f, int status);

int msg_shutdown(struct peer *peer, struct file *f, int status);

int store_chunk(msg_t *msg, struct file *f, unsigned int chunk);

int *tree_bf2df(int nb, int degree, int start);

int *tree_partition(int nb, int degree);

#endif /* PROTOCOL_H */
