/*
 * File:
 *   source.h
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Represents a data source.
 *
 * Copyright (c) 2004-2005.
 */

#ifndef SOURCE_H
#define SOURCE_H

#include "common.h"
#include "file.h"
#include "message.h"
#include "network.h"
#include "peer.h"

/* ################################################################### *
 * TYPEDEFS
 * ################################################################### */

struct config;
struct ssh_connection;
struct arg_list;

/* A destination, as seen by the source */
typedef struct destination {
  peer_t peer;                  /* Network connection (bi-directional) */
  ssh_connection_t *ssh;        /* SSH connection used for launching remote process */
  char *host;                   /* Host name of remote peer */
  int port;                     /* Port number of remote peer */
  char *path;                   /* File path for storing content at remote peer */
} destination_t;

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

void init_source(char *src, char **dst, int nb_dst, struct arg_list *ssh_args, struct config *config);

void terminate_source();

void close_destination(destination_t *dst);

#endif /* SOURCE_H */
