/*
 * File:
 *   stat.c
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Statistic utilities.
 *
 * Copyright (c) 2004-2005.
 */

#include <math.h>

#include "stat.h"

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

/* ======================================================================
 * init_samples
 */
void init_samples(samples_t* s)
{
  s->samples = 0;
  s->mean = 0.0;
  s->variance = 0.0;
}

/* ======================================================================
 * add_sample
 */
void add_sample(samples_t* s, double d)
{
  double previous_mean = s->mean;
  s->samples++;
  s->mean = s->mean + (d - s->mean) / (double)s->samples;
  s->variance = s->variance + (d - previous_mean) * (d - s->mean);
}

/* ======================================================================
 * samples_count
 */
int samples_count(samples_t* s)
{
  return s->samples;
}

/* ======================================================================
 * samples_mean
 */
double samples_mean(samples_t* s)
{
  return s->mean;
}

/* ======================================================================
 * samples_var
 */
double samples_variance(samples_t* s)
{
  return s->variance;
}

/* ======================================================================
 * samples_stddev
 */
double samples_stddev(samples_t* s)
{
  return sqrt(s->variance);
}
