/*
 * File:
 *   util.c
 * Author(s):
 *   Pascal Felber <Pascal.Felber@unine.ch>
 * Description:
 *   Utility functions.
 *
 * Copyright (c) 2004-2005.
 */

#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "util.h"

#include "log.h"

/* ################################################################### *
 * FUNCTIONS
 * ################################################################### */

/* ======================================================================
 * xread
 */
ssize_t xread(int fd, void *p, size_t n)
{
  char *s = p;
  ssize_t res, pos = 0;

  while (n > pos) {
    /* read returns 0 when EOF is reached */
    res = read(fd, s + pos, n - pos);
    switch (res) {
     case -1:
       if (errno == EINTR || errno == EAGAIN)
         continue;
     case 0:
       return res;
     default:
       pos += res;
    }
  }
  return pos;
}

/* ======================================================================
 * xwrite
 */
ssize_t xwrite(int fd, void *p, size_t n)
{
  char *s = p;
  ssize_t res, pos = 0;

  while (n > pos) {
    res = write(fd, s + pos, n - pos);
    switch (res) {
     case -1:
       if (errno == EINTR || errno == EAGAIN)
         continue;
     case 0:
       return res;
     default:
       pos += res;
    }
  }
  return pos; 
}

/* ======================================================================
 * xcalloc
 */
void *xcalloc(size_t count, size_t size)
{
  void *p = (void *)calloc(count, size);
  if (p == NULL)
    fatal(1, "%s: calloc\n", __func__);
 return p;
}

/* ======================================================================
 * xmalloc
 */
void *xmalloc(size_t size)
{
  void *p = (void *)malloc(size);
  if (p == NULL)
    fatal(1, "%s: malloc\n", __func__);
 return p;
}

/* ======================================================================
 * xrealloc
 */
void *xrealloc(void *ptr, size_t size)
{
  void *p = (void *)realloc(ptr, size);
  if (p == NULL)
    fatal(1, "%s: realloc\n", __func__);
  return p;
}

/* ======================================================================
 * xstrdup
 */
char *xstrdup(const char *str)
{
  char *s = strdup(str);
  if (s == NULL)
    fatal(1, "%s: strdup\n", __func__);
  return s;
}
